"""SNAPtoolbelt Firmware Commands"""
import logging
import sys
from textwrap import dedent
from argparse import RawDescriptionHelpFormatter

from arghandler import ArgumentHandler
from snap_modules.snap_modules import get_cores, get_parts, get_supported_cores_for_part

from snaptoolbelt.util import exit_with_error

log = logging.getLogger(__name__)


def firmware(parser, context, args):
    """SNAP Firmware commands"""
    handler = ArgumentHandler(
        usage="snap [global options] firmware [-h] (list) [options]",
        description=dedent("""
        SNAPtoolbelt Firmware Commands:
          list: Lists firmware versions
        """),
        formatter_class=RawDescriptionHelpFormatter,
    )

    # Add subcommands here.
    handler.set_subcommands({
        'list': firmware_list,
    })

    handler.run(args, context_fxn=lambda x: context)


def firmware_list(parser, context, args):
    """Listing Configuration-related commands"""
    handler = ArgumentHandler(
        usage="snap [global options] firmware list [-h] [module]",
        description="List firmware versions all or for a SNAP module",
    )

    all_modules = get_parts()

    handler.add_argument('module',
                         default=None,
                         nargs='?',
                         help="Module name i.e. {}".format(', '.join(all_modules)))

    args, rest = handler.parse_known_args(args)

    if rest:
        log.error("Unexpected arguments to list firmware command: %r", rest)
        sys.exit(1)

    log.info("Listing firmwares")
    all_firmwares = None

    if not args.module:
        all_firmwares = get_cores()
    else:
        if args.module in all_modules:
            all_firmwares = get_supported_cores_for_part(args.module)
        else:
            exit_with_error("Unknown SNAP module: {0}".format(args.module),
                            "Valid SNAP modules are {0}".format(', '.join(all_modules)))

    map(context.output, all_firmwares)
